<template>
	<div class="app-container">
		<div class="search_form">
			<el-form :model="searchForm" :inline="true">
				<el-form-item label="客户名称:">
					<el-input v-model="searchForm.customerName" placeholder="请输入" clearable prefix-icon="Search"
										@change="handleQuery" />
				</el-form-item>
				<el-form-item label="项目名称:">
					<el-input v-model="searchForm.projectName" placeholder="请输入" clearable prefix-icon="Search"
										@change="handleQuery" />
				</el-form-item>
				<el-form-item label="排产日期:">
					<el-date-picker v-model="searchForm.entryDate" value-format="YYYY-MM-DD" format="YYYY-MM-DD" type="daterange"
													placeholder="请选择" clearable @change="changeDaterange" />
				</el-form-item>
				<el-form-item label="状态:">
					<el-select v-model="searchForm.status" placeholder="请选择状态" style="width: 140px" clearable>
						<el-option label="待生产" :value="1"></el-option>
						<el-option label="已报工" :value="3"></el-option>
						<el-option label="生产中" :value="2"></el-option>
					</el-select>
				</el-form-item>
				<el-form-item>
					<el-button type="primary" @click="handleQuery">搜索</el-button>
				</el-form-item>
			</el-form>
		</div>
		<div class="table_list">
			<div style="text-align: right" class="mb10">
				<el-button type="primary" @click="openForm('add')">生产报工</el-button>
			</div>
			<PIMTable
				rowKey="id"
				:column="tableColumn"
				:tableData="tableData"
				:page="page"
				:isSelection="true"
				:expandRowKeys="expandedRowKeys"
				@expand-change="expandChange"
				@selection-change="handleSelectionChange"
				:tableLoading="tableLoading"
				@pagination="pagination"
				:total="page.total"
			>
				<template #expand="{ row }">
					<el-table
						:data="expandData"
						border
						show-summary
						:summary-method="summarizeMainTable"
						v-loading="childrenLoading"
					>
						<el-table-column
							align="center"
							label="序号"
							type="index"
							width="60"
						/>
						<el-table-column label="本次生产数量" prop="finishedNum" align="center">
							<template #default="scope">
								<el-input-number :step="0.01" :min="0" style="width: 100%"
																 v-model="scope.row.finishedNum"
																 :disabled="!scope.row.editType"
																 :precision="2"
																 placeholder="请输入"
																 clearable
																 @change="changeNum(scope.row)"
								/>
							</template>
						</el-table-column>
<!--						<el-table-column label="待生产数量" prop="pendingNum" width="240" align="center"></el-table-column>-->
						<el-table-column label="生产人" prop="schedulingUserId" width="240">
							<template #default="scope">
								<el-select
									v-model="scope.row.schedulingUserId"
									placeholder="选择人员"
									:disabled="!scope.row.editType"
									style="width: 100%;"
								>
									<el-option
										v-for="user in userList"
										:key="user.userId"
										:label="user.nickName"
										:value="user.userId"
									/>
								</el-select>
							</template>
						</el-table-column>
						<el-table-column label="生产日期" prop="schedulingDate" width="240">
							<template #default="scope">
								<el-date-picker
									v-model="scope.row.schedulingDate"
									type="date"
									:disabled="!scope.row.editType"
									placeholder="请选择日期"
									value-format="YYYY-MM-DD"
									format="YYYY-MM-DD"
									clearable
									style="width: 100%"
								/>
							</template>
						</el-table-column>
						<el-table-column label="操作" width="60">
							<template #default="scope">
								<el-button
									link
									type="primary"
									size="small"
									@click="changeEditType(scope.row)"
									v-if="!scope.row.editType"
									:disabled="scope.row.parentStatus === 3"
								>编辑</el-button
								>
								<el-button
									link
									type="primary"
									size="small"
									@click="saveReceiptPayment(scope.row)"
									v-if="scope.row.editType"
								>保存</el-button
								>
							</template>
						</el-table-column>
					</el-table>
				</template>
			</PIMTable>
		</div>
		<form-dia ref="formDia" @close="handleQuery"></form-dia>
	</div>
</template>

<script setup>
import {onMounted, ref} from "vue";
import FormDia from "@/views/productionManagement/productionReporting/components/formDia.vue";
import {staffJoinDel, staffJoinListPage} from "@/api/personnelManagement/onboarding.js";
import {ElMessageBox} from "element-plus";
import dayjs from "dayjs";
import {
	productionReportUpdate,
	workListPage,
	workListPageById
} from "@/api/productionManagement/productionReporting.js";
import {userListNoPageByTenantId} from "@/api/system/user.js";

const data = reactive({
	searchForm: {
		staffName: "",
		entryDate: [
			dayjs().format("YYYY-MM-DD"),
			dayjs().add(1, "day").format("YYYY-MM-DD"),
		], // 录入日期
		entryDateStart: dayjs().format("YYYY-MM-DD"),
		entryDateEnd: dayjs().add(1, "day").format("YYYY-MM-DD"),
	},
});
const { searchForm } = toRefs(data);
const expandedRowKeys = ref([]);
const expandData = ref([]);
const userList = ref([])
const tableColumn = ref([
	{
		type: "expand",
		dataType: "slot",
		slot: "expand",
	},
	{
		label: "状态",
		prop: "status",
		dataType: "tag",
		formatData: (params) => {
			if (params == 3) {
				return "已报工";
			} else if (params == 1) {
				return "待生产";
			} else {
				return '生产中';
			}
		},
		formatType: (params) => {
			if (params == 3) {
				return "success";
			} else if (params == 1) {
				return "primary";
			} else {
				return 'warning';
			}
		},
	},
	{
		label: "排产日期",
		prop: "schedulingDate",
		width: 120,
	},
	{
		label: "排产人",
		prop: "schedulingUserName",
	},
	{
		label: "合同号",
		prop: "salesContractNo",
		width: 200,
	},
	{
		label: "客户合同号",
		prop: "customerContractNo",
		width: 200,
	},
	{
		label: "客户名称",
		prop: "customerName",
		width: 200,
	},
	{
		label: "项目名称",
		prop: "projectName",
		width:300
	},
	{
		label: "产品大类",
		prop: "productCategory",
		width: 150,
	},
	{
		label: "规格型号",
		prop: "specificationModel",
		width: 150,
	},
	{
		label: "单位",
		prop: "unit",
	},
	{
		label: "工序",
		prop: "process",
	},
	{
		label: "排产数量",
		prop: "schedulingNum",
		width: 100,
	},
	{
		label: "生产数量",
		prop: "finishedNum",
		width: 100,
	},
	{
		label: "待生产数量",
		prop: "pendingFinishNum",
		width: 100,
	},
]);
const tableData = ref([]);
const selectedRows = ref([]);
const tableLoading = ref(false);
const childrenLoading = ref(false);
const page = reactive({
	current: 1,
	size: 100,
	total: 0,
});
const formDia = ref()
const { proxy } = getCurrentInstance()

// 查询列表
/** 搜索按钮操作 */
const handleQuery = () => {
	page.current = 1;
	getList();
};
const changeDaterange = (value) => {
	if (value) {
		searchForm.value.entryDateStart = value[0];
		searchForm.value.entryDateEnd = value[1];
	} else {
		searchForm.value.entryDateStart = undefined;
		searchForm.value.entryDateEnd = undefined;
	}
	handleQuery();
};
const pagination = (obj) => {
	page.current = obj.page;
	page.size = obj.limit;
	getList();
};
const getList = () => {
	tableLoading.value = true;
	const params = { ...searchForm.value, ...page };
	params.entryDate = undefined
	expandedRowKeys.value = []
	workListPage(params).then(res => {
		tableLoading.value = false;
		tableData.value = res.data.records.map(item => ({
			...item,
			pendingFinishNum: (Number(item.schedulingNum) || 0) - (Number(item.finishedNum) || 0)
		}));
		page.total = res.data.total;
	}).catch(err => {
		tableLoading.value = false;
	})
};
// 展开行
const expandChange = (row, expandedRows) => {
	userListNoPageByTenantId().then((res) => {
		userList.value = res.data;
	});
	if (expandedRows.length > 0) {
		nextTick(() => {
			expandedRowKeys.value = [];
			try {
				childrenLoading.value = true;
				workListPageById({ id: row.id }).then((res) => {
					childrenLoading.value = false;
					const index = tableData.value.findIndex((item) => item.id === row.id);
					if (index > -1) {
						expandData.value = res.data.map(item => ({
							...item,
							pendingNum: (Number(item.schedulingNum) || 0) - (Number(item.finishedNum) || 0),
							parentStatus: row.status // 新增父表状态
						}));
					}
					expandedRowKeys.value.push(row.id);
				});
			} catch (error) {
				childrenLoading.value = false;
				console.log(error);
			}
		})
	} else {
		expandedRowKeys.value = [];
	}
};
const changeNum = (row) => {
	// 找到父表格数据
	const parentRow = tableData.value.find(item => item.id === expandedRowKeys.value[0]);
	// 计算所有子表格 finishedNum 的总和
	const totalFinishedNum = expandData.value.reduce((sum, item) => sum + (Number(item.finishedNum) || 0), 0);
	// 父表格的排产数量
	const schedulingNum = parentRow ? Number(parentRow.schedulingNum) : 0;
	
	if (totalFinishedNum > schedulingNum) {
		// 回退本次输入
		row.finishedNum = schedulingNum - (totalFinishedNum - Number(row.finishedNum));
		proxy.$modal.msgWarning('所有本次生产数量之和不可大于排产数量');
	}
	row.pendingNum = row.schedulingNum - row.finishedNum;
}
// 编辑修改状态
const changeEditType = (row) => {
	row.editType = !row.editType;
};
// 保存记录
const saveReceiptPayment = (row) => {
	productionReportUpdate(row).then((res) => {
		row.editType = !row.editType;
		getList();
		proxy.$modal.msgSuccess("提交成功");
	});
};
// 表格选择数据
const handleSelectionChange = (selection) => {
	selectedRows.value = selection;
};
const summarizeMainTable = (param) => {
	return proxy.summarizeTable(param, [
		"finishedNum"
	]);
};
// 打开弹框
const openForm = (type, row) => {
	if (selectedRows.value.length !== 1) {
		proxy.$message.error("请选择一条数据");
		return;
	}
	if (selectedRows.value[0].pendingFinishNum == 0) {
		proxy.$message.warning("无需再报工");
		return;
	}
	nextTick(() => {
		const rowInfo = type === 'add' ? selectedRows.value[0] : row
		formDia.value?.openDialog(type, rowInfo)
	})
};

// 删除
const handleDelete = () => {
	let ids = [];
	if (selectedRows.value.length > 0) {
		ids = selectedRows.value.map((item) => item.id);
	} else {
		proxy.$modal.msgWarning("请选择数据");
		return;
	}
	ElMessageBox.confirm("选中的内容将被删除，是否确认删除？", "导出", {
		confirmButtonText: "确认",
		cancelButtonText: "取消",
		type: "warning",
	})
		.then(() => {
			staffJoinDel(ids).then((res) => {
				proxy.$modal.msgSuccess("删除成功");
				getList();
			});
		})
		.catch(() => {
			proxy.$modal.msg("已取消");
		});
};
// 导出
const handleOut = () => {
	ElMessageBox.confirm("选中的内容将被导出，是否确认导出？", "导出", {
		confirmButtonText: "确认",
		cancelButtonText: "取消",
		type: "warning",
	})
		.then(() => {
			proxy.download("/staff/staffJoinLeaveRecord/export", {staffState: 1}, "人员入职.xlsx");
		})
		.catch(() => {
			proxy.$modal.msg("已取消");
		});
};
onMounted(() => {
	getList();
});
</script>

<style scoped></style>
